<?php
namespace MyMenu\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Session\Session;
use MyMenu\Libraries\KeyValueStorage;

class Password
{

    private $crudService;
    private $sessionManager;
    private $smtp;

    public function __construct($crudService, Session $sessionManager, $smtp)
    {
        $this->crudService = $crudService;
        $this->sessionManager = $sessionManager;
        $this->smtp = $smtp;
    }

    public function sendPwRecoverToken($request, $response)
    {
        if ($request->getRow() && $request->getRow()['email']) {
            $email = $request->getRow()['email'];

            $conditions = array(
                array(
                    "columnName" => "UPPER_DSEMAILUSUARIOAPP",
                    "operator" => "=",
                    "value" => strtoupper($email),
                ),
            );

            $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $conditions, 1, 1);

            if (sizeof($resDataSet->getRows()) == 1) {
                $userRes = $resDataSet->getRows()[0];

                $user = $userRes['NRUSUARIOAPP'];
                $passwd = $userRes['DSSENHAUSUARIOAPP'];

                $token = '';
                for ($i = 0; $i < 6; $i++) {
                    $token = $token . rand(0, 9);
                }

                $updateRequest = new DTO\Request\DataSet($resDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $rowsToUpdate = $this->crudService->getRowsToSave($updateRequest, 'usuarioapp');
                if (sizeof($rowsToUpdate) == 1) {
                    $rowsToUpdate[0]["CDVERIFICASENHA"] = $token;
                    $this->crudService->save('usuarioapp', $rowsToUpdate);
                    $msg = $this->buildRecoverTokenMail($token);
                    $this->smtp->setSubject("Redefinição de senha TecFood MyMenu");
                    $this->smtp->sendMail($msg, $email);
                } else {
                    $response->setError(new Error("Erro ao executar a operação, tente novamente.", 500));
                }
            } else {
                $response->setError(new Error("Email não cadastrado.", 500));
            }
        } else {
            $response->setError(new Error("Erro ao executar a operação, tente novamente.", 500));
        }
    }

    private function buildRecoverTokenMail($token)
    {
        $title = "Etapa de verificação";
        $mailTemplate = file_get_contents(__DIR__ . "/../Templates/mail.html");
        $content = mb_convert_encoding($mailTemplate, 'UTF-8', mb_detect_encoding($mailTemplate, 'UTF-8, ISO-8859-1', true));
        $footerMessage = "Email automático enviado pelo TecFood MyMenu.<br>Não responder.";

        $search = array(
            "%title%",
            "%token%",
            "%footerMessage%",
        );
        $replace = array(
            $title,
            $token,
            $footerMessage,
        );
        return str_replace($search, $replace, $content);
    }

    public function resetPassword($request, $response)
    {
        if ($request->getRow() && $request->getRow()['email']) {
            $email = $request->getRow()['email'];
            $token = $request->getRow()['token'];

            $conditions = array(
                array(
                    "columnName" => "UPPER_DSEMAILUSUARIOAPP",
                    "operator" => "=",
                    "value" => strtoupper($email),
                ),
                array(
                    "columnName" => "CDVERIFICASENHA",
                    "operator" => "=",
                    "value" => $token,
                ),
            );

            $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $conditions, 1, 1);

            if (sizeof($resDataSet->getRows()) == 1) {
                $userRes = $resDataSet->getRows()[0];
                $user = $userRes['NRUSUARIOAPP'];

                $newPw = $request->getRow()['newPw'];

                $this->updatePassword($user, $newPw, $request, $response);
            } else {
                $response->setError(new Error("Token inválido para o email informado.", 500));
            }
        } else {
            $response->setError(new Error("Erro ao executar a operação, tente novamente.", 500));
        }
    }

    public function changePassword($request, $response)
    {
        $pwData = $request->getRow();

        $user = KeyValueStorage::get("NRUSUARIOAPP");

        $oldPw = $pwData['OLD_PW'];
        $newPw = $pwData['NEW_PW'];

        $md5OldPw = md5($oldPw);

        $conditions = array(
            array(
                "columnName" => "NRUSUARIOAPP",
                "operator" => "=",
                "value" => $user,
            ),
            array(
                "columnName" => "DSSENHAUSUARIOAPP",
                "operator" => "=",
                "value" => $md5OldPw,
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
        );

        $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp', $conditions, 1, 1);

        if (sizeof($resDataSet->getRows()) == 1) {
            $this->updatePassword($user, $newPw, $request, $response);
        } else {
            $response->setError(new Error("Senha inválida", 500));
        }
    }

    private function updatePassword($user, $newPw, $request, $response)
    {
        $conditions = array(
            array(
                "columnName" => "NRUSUARIOAPP",
                "operator" => "=",
                "value" => $user,
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
        );
        $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp', $conditions, 1, 1);

        if (sizeof($resDataSet->getRows()) == 1) {
            $updateRequest = new DTO\Request\DataSet($resDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $rowsToUpdate = $this->crudService->getRowsToSave($updateRequest, 'usuarioapp');
            if (sizeof($rowsToUpdate) == 1) {
                $md5NewPw = md5($newPw);
                $rowsToUpdate[0]["DSSENHAUSUARIOAPP"] = $md5NewPw;
                $rowsToUpdate[0]["CDVERIFICASENHA"] = null;
                $this->crudService->save('usuarioapp', $rowsToUpdate);
            } else {
                $response->setError(new Error("Erro ao executar a operação, tente novamente.", 500));
            }
        } else {
            $response->setError(new Error("Erro ao executar a operação, tente novamente.", 500));
        }
    }
}
