function QuestionaryController(ScreenService, General, metaDataFactory) {
    this.onEnter = function (questionaryWidget) {
        if (questionaryWidget.container.questionary) {
            if (questionaryWidget.container.questionary.twiceCallCorrection) {
                return false; // correcao do bug de chamada duplicada do onEnter
            } else {
                questionaryWidget.container.questionary.twiceCallCorrection = true;
            }

            questionaryWidget.fields = [];

            var editWidget = function(widget){
                widget.edit();
                widget.template = widget.templateToForm;
            };
            var viewWidget = function (widget) {
                widget.view();
                widget.template = widget.templateToView;
            };

            var modeDecode = {
                "Disponível": editWidget,
                "Em andamento": editWidget,
                "Finalizado": viewWidget
            };

            questionaryWidget.currentRow = questionaryWidget.container.questionary;
            var currentRow = questionaryWidget.currentRow;
            
            var modeFunc = modeDecode[currentRow.STATUS];
            modeFunc(questionaryWidget);
            
            questionaryWidget.container.label = currentRow.NMQUESTIONARIO;

            var filter = [
                {
                    "name": "NRQUESTIONARIO",
                    "operator": "=",
                    "value": currentRow.NRQUESTIONARIO
                },
                {
                    "name": "NRPROGRAMACAOPESQ",
                    "operator": "=",
                    "value": currentRow.NRPROGRAMACAOPESQ
                },
                {
                    "name": "NREXECUCAOPESQ",
                    "operator": "=",
                    "value": currentRow.NREXECUCAOPESQ
                }
            ];
            General.callRestEngine('/getQuestionaryExec', 'FilterData', filter).then(function(response){
                if (response && response.dataset) {
                    var questoes = response.dataset.questionario_questoes || [];
                    var respostas = response.dataset.questionario_respostas_execucao || [];

                    questionaryWidget.fields = [];
                    questionaryWidget.fieldSequence = 0;
                    questoes.forEach(function(questao) {
                        var fieldName = 'NRQUESTAO' + questao.NRQUESTAO;                        
                        var newField = {
                            "isVisible": true,
                            "name": fieldName,
                            "label": questao.TXQUESTAO,
                            "template": getFieldTemplate(questao),
                            "class": 12,
                            "validations": getFieldValidations(questao),
                            "treatAnswer": getFunctionForTreatAnswer(questao),
                            "dataSource": {
                                "data": []
                            },
                            "valueField": "NRRESPOSTA",
                            "descriptionField": "DSRESPOSTA"
                        };
                        metaDataFactory.fieldFactory(newField, questionaryWidget, true);
                    });
                    if (questionaryWidget.currentRow.STATUS != "Finalizado") {
                        var finalizarField = {
                            "name": "finalizar",
                            "label": "Finalizar",
                            "isVisible": true,
                            "template": "field/button.html",
                            "class": 12
                        };
                        metaDataFactory.fieldFactory(finalizarField, questionaryWidget, true);
                    }

                    respostas.forEach(function(resposta) {
                        var fieldName = 'NRQUESTAO' + resposta.NRQUESTAO;
                        var questaoField = questionaryWidget.getField(fieldName);
                        questaoField.treatAnswer(questaoField, resposta);
                    });
                }
            });
        } else {
            ScreenService.openWindow("home");
        }
    };

    var getFieldValidations = function (questao) {
        return (questao.IDQUESTAOOBRIGA == 'S') ? { "required": true } : null;
    };
    var getFieldTemplate = function (questao) {
        var idtipoquestao = questao.IDTIPOQUESTAO;
        var idopcaoresposta = questao.IDOPCAORESPOSTA;

        if (idtipoquestao === "ABERTA_TEXTO") {
            return "field/text-edit.html";
        } else if (idtipoquestao === "FECHADA") {
            if (idopcaoresposta === "UNICA") {
                return "field/select-native.html";
            } else if (idopcaoresposta === "MULTIPLAS") {
                return "field/select-multiple.html";                
            }
        }
        return "field/text-edit.html";
    };
    var getFunctionForTreatAnswer = function (questao) {
        var idtipoquestao = questao.IDTIPOQUESTAO;
        var idopcaoresposta = questao.IDOPCAORESPOSTA;

        if (idtipoquestao === "ABERTA_TEXTO") {
            return function (questaoField, resposta) {
                questaoField.setValue(resposta.DSRESPOSTA);
            };
        } else if (idtipoquestao === "FECHADA") {
            if (idopcaoresposta === "UNICA") {
                return function (questaoField, resposta) {
                    if (resposta.IDRESPOSTA === "S") {
                        questaoField.setValue(resposta.NRRESPOSTA);
                    }
                    questaoField.dataSource.data.push(resposta);
                };
            } else if (idopcaoresposta === "MULTIPLAS") {
                return function (questaoField, resposta) {
                    if (resposta.IDRESPOSTA === "S") {
                        var currentRow = questaoField.widget.currentRow;
                        var fieldName = questaoField.name;
                        if (Array.isArray(currentRow[fieldName])) {
                            currentRow[fieldName].push(resposta.NRRESPOSTA);
                        } else {
                            currentRow[fieldName] = [resposta.NRRESPOSTA];
                        }
                        resposta.checked = true;
                    }
                    questaoField.dataSource.data.push(resposta);
                };
            }
        }
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('QuestionaryController', QuestionaryController);
});